@@ -325,7 +325,7 @@ bool symbol::is_equal_same_type(const basic & other) const
 long symbol::calchash() const
 {
   // Original code
-    hashvalue = golden_ratio_hash((p_int)tinfo() ^ serial);
+    hashvalue = golden_ratio_hash((intptr_t)tinfo() ^ serial);
     setflag(status_flags::hash_calculated);
     return hashvalue;
 }
